<?php
/*

  OTHELLO -- The Game of Dramatic Reversals

  written by Bert Halstead
  modified for BDS C by Leor Zolman
  modified for PHP by Masaki Oba(6 DEC 2016)
  (PHP version might commands does not work.no check)

This program is a good example of:

  a) structured, heirarchical function organization
  b) arrays as formal parameters
  c) use of the "qsort" library function

   Object of the game is for two players to alternate
placing their marker someplace on an 8 by 8 grid, so that
at least one of the opponent's pieces becomes surrounded
by the moving player's peices -- causing the flanked pieces
to flip 'color' and belong to the moving player. After 60
moves have been played (or if no player has a legal move left),
the player with the most of his own pieces on the board wins.

   The playing pieces are '*' and '@'. You may choose to play
either '*' or '@' for the first game; thereafter, you and the
computer will alternate going first for each game. Whoever
goes first always plays `*'.

   You enter a move as a two digit number, each digit being
from 1 to 8, first digit representing row and second representing
column. For example: if playing '*', your first move might be '46',
meaning 4th row down, 6th position across.

   As an alternative to entering a move, one of the following
commands may be typed:

  g  causes computer to play both sides until game
    is over

  a  causes computer to print out an analysis of
    each of your possible moves. A letter from A
    to Z will appear at each of your legal move
    positions, where A is the machine's opinion
    of an excellant move and Z is a real loser.

  hn  sets handicap. n is 1,2,3, or 4. If n is
    positive, gives n free pieces to the computer.
    If n is negative, gives YOU the free peices.

  f  forfeit the current move. This happens
    automatically if you have no legal moves.

  q  quit the current game.

  b  prints out board again.

  s  prints out the score, and tells who is winning.

*/
/*
struct mt{
  int x;
  int y;
  int c;
  int s;
};
*/
for($i=0;$i<64;++$i){
  $mt[$i] = new stdClass;
  $mt[$i]->x = 0;
  $mt[$i]->y = 0;
  $mt[$i]->c = 0;
  $mt[$i]->s = 0;

}

$mt_tmp = new stdClass;
$mt_tmp->x = 0;
$mt_tmp->y = 0;
$mt_tmp->c = 0;
$mt_tmp->s = 0;

$BLACK = '*';
$WHITE = '@';
$EMPTY = '-';


$handicap = 0;
$selfplay = ' ';    /* true if computer playing with itself */
//int h[4][2];    /* handicap position table */
for($i=0;$i<4;++$i){
  for($j=0;$j<2;++$j){
    $h[$i][$j] = 0;
  }
}

//char a[8][8];
//char b[8][8];
for($i=0;$i<8;++$i){
  for($j=0;$j<8;++$j){
    $a[$i][$j] = $EMPTY;
    $b[$i][$j] = $EMPTY;
  }
}

$mine = ' '; $his = ' ';    /* who has black (*) and white (@) in current game */
$mefirst = 0;    /* true if computer goes first in current game */

function toupper($c)
{
  return strtoupper($c);
}

function getchar()
{
  $ct;
  $c = chr(0);
  while(1){
    $ct = fgetc(STDIN);
    if(ord($ct) <= 0){
      return 0;
    }
    if($ct == "\n"){
      break;
    }
    if($ct != "\r"){
      $c = $ct;
    }
  }
  return $c;
}

function putchar($c)
{
  print($c);
}

function clrbrd(&$b)
{
  global $BLACK;
  global $WHITE;
  global $EMPTY;

  for ($i=0; $i<8; $i++)
    for ($j=0; $j<8; $j++)
      $b[$i][$j]= $EMPTY;
  $b[3][3] = $b[4][4] = $BLACK;
  $b[3][4] = $b[4][3] = $WHITE;
}

function prtbrd($b)
{
  printf("   1 2 3 4 5 6 7 8\n");
  for ($i=0; $i<8; $i++) {
    printf("%2d",$i+1);
    for ($j=0; $j<8; $j++) {
      putchar(' ');
      putchar($b[$i][$j]);
     }
    putchar("\n");
   }
  putchar("\n");
}

function chkmv1($b,$p,$x,$y,$m,$n)
{
  global $EMPTY;

  $k=0;
  while (($x += $m) >= 0 && $x < 8 && ($y += $n) >= 0 && $y<8)
  {
    if ($b[$x][$y]==$EMPTY){
      return 0;
    }
    if ($b[$x][$y]== $p ){
      return $k;
    }
    if ($x==0 || $x==7 || $y==0 || $y==7){
      $k += 10;
    }
    else{
      $k++;
    }
  }
  return 0;
}

function chkmov($b,$p,$x,$y)
{
  global $EMPTY;
  if ($b[$x][$y] != $EMPTY){
    return 0;
  }
  return  chkmv1($b,$p,$x,$y,0,1) + chkmv1($b,$p,$x,$y,1,0) +
    chkmv1($b,$p,$x,$y,0,-1)+ chkmv1($b,$p,$x,$y,-1,0)+
    chkmv1($b,$p,$x,$y,1,1) + chkmv1($b,$p,$x,$y,1,-1)+
    chkmv1($b,$p,$x,$y,-1,1)+ chkmv1($b,$p,$x,$y,-1,-1);
}

function chkmvs($b,$p)
{
  $k=0;
  for ($i=0; $i<8; $i++) for ($j=0; $j<8; $j++)
    $k += chkmov($b,$p,$i,$j);
  return $k;
}

function getstring()
{
  if(($buf = fgets(STDIN,2000)) !== false){
    return strtoupper(trim($buf));
  }
  return "\n";
}

function getmov(&$game_i,&$game_j)
{
  global $EMPTY;
  global $selfplay;

  if ($selfplay == 'G') {
    //if (!kbhit()) return 'G';
    $selfplay = ' ';
    getchar();
  }
  printf("Move: ");
  while(1){
    $c=getstring();
    $s0 = substr($c,0,1);
    if($c == "\n"){
      printf("Move?  ");
      continue;
    }
    else
    if($c == 'G'){
      $selfplay='G';
      return 'G';
    }
    else
    if($c == 'B' || $c == 'S' || $c == 'Q' ||
      $c == 'F' || $c == 'A'){
      return $c;
    }
    else
    if($s0 == 'H'){
      $s1 = substr($c,1,1);
      $s2 = substr($c,2,1);
      if (ord($s1)<ord('1') || ord($s1)>ord('4'))
        goto flush;
      if($s2 === false){
        $game_i = -(ord($c)-ord('0'));
      }
      else{
        $game_i = (ord($c)-ord('0'));
      }
      return 'H';
    }
/*
    case 4:
printf("case 4\n");
      return $c;
*/
    else{
      $s1 = substr($c,0,1);
      $s2 = substr($c,1,1);
      if (ord($s1)<ord('1') || ord($s1)>ord('8')) goto flush;
      if (ord($s2)<ord('1') || ord($s2)>ord('8')) goto flush;
       $game_i = ord($s1) - ord('1');
       $game_j = ord($s2) - ord('1');
       return 'M';
    }
    flush:  
      printf ("Huh?? ");
  }
}

function cntbrd($b,$p)
{
  $j = 0;

  for($x=0;$x<8;++$x){
    for($y=0;$y<8;++$y){
      if($b[$x][$y] == $p){
        ++$j;
      }
    }
  }
  return ($j);
}

function prtscr($b)
{
  global $his;
  global $mine;
  printf("%1d-%1d",$i = cntbrd($b,$his), $j=cntbrd($b,$mine));
  return $i-$j;
}

function putmv1(&$b,$p,$x,$y,$m,$n)
{
  global $EMPTY;
  while (($x += $m) >= 0 && $x<8 && ($y += $n)>=0 && $y<8) {
    if ($b[$x][$y] == $EMPTY || $b[$x][$y] == $p) return;
    $b[$x][$y] = $p;
   }
}

function putmov(&$b,$p,$x,$y)
{
  $b[$x][$y] = $p;
  for ($i= -1; $i<=1; $i++) for ($j= -1; $j<=1; $j++) {
    if (($i != 0 || $j!=0)&&chkmv1($b,$p,$x,$y,$i,$j)>0)
      putmv1($b,$p,$x,$y,$i,$j);
   }
}

function cpybrd(&$a,$b)
{
  for($x=0;$x<8;++$x){
    for($y=0;$y<8;++$y){
      $a[$x][$y] = $b[$x][$y];
    }
  }
}

function notak2($b,$p,$o,$e,$x,$y,$m,$n)
{
  $x += $m; $y +=$n;
  if ($x>=0 && $x<=7 && $y>=0 && $y<=7)
    while($b[$x][$y] == 0) {
     $x += $m; $y+=$n;
     if ($x<0 || $x>7 || $y<0 || $y>7 || $b[$x][$y]==$e)
      return $o;
     }
  while ($x>=0 && $x<=7 && $y>=0 && $y<=7 && $b[$x][$y]==$p)
      { $x +=$m; $y+=$n; }
  if ($x<0 || $x>7 || $y<0 || $y>7) return $p;
  return $b[$x][$y];
}

function notak1($b,$p,$o,$e,$x,$y,$m,$n)
{
  $c1 = notak2($b,$p,$o,$e,$x,$y,$m,$n);
  $c2 = notak2($b,$p,$o,$e,$x,$y,-$m,-$n);
  return !($c1==$o && $c2==$e || $c1==$e && $c2==$o);
}

function notake($b,$p,$o,$e,$x,$y)
{
  return notak1($b,$p,$o,$e,$x,$y,0,1)&&
    notak1($b,$p,$o,$e,$x,$y,1,1)&&
    notak1($b,$p,$o,$e,$x,$y,1,0)&&
    notak1($b,$p,$o,$e,$x,$y,1,-1);
}

function s_move($b,$p,$o,$e,$i,$j)
{
  global $a;
  cpybrd($a,$b);
  putmov($a,$p,$i,$j);
  $side = 0;
  if ($i==0 || $i==7) $side++;
  if ($j==0 || $j==7) $side++;
  $s = 0;
  $ok = 0;
  if ($side==2 || notake($b,$p,$o,$e,$i,$j)) $ok++;
  $oside = 0;
  for ($k=0; $k<8; $k++) for($l=0; $l<8; $l++)
   {
    $c=chkmov($a,$o,$k,$l);
    if ($c==0) continue;
    $dkl = 1;
    if ($k==0 || $k==7) { $dkl+=2; $oside|=4;}
    if ($l==0 || $l==7) {$dkl+=2; $oside|=4; }
    if ($dkl==5) {$dkl = 10; $oside |= 16; }
      else if (!notake($a,$o,$p,$e,$k,$l))
          continue;
    $oside |= 1;
    $s -= $dkl;
    if ($c>=10) { $s -= 4; $oside |= 8; }
    }
  if ($s< -$oside) $s= -$oside;
  if ($side>0) return $s+$side-7+10*$ok;
  if ($i==1 || $i==6) {$s--; $side++;}
  if ($j==1 || $j==6) {$s--; $side++;}
  if ($side>0) return $s;
  if ($i==2 || $i==5) $s++;
  if ($j==2 || $j==5) $s++;
  return $s;
}


function fillmt($b,$p,$o,$e)
{
  global $mt;

  $k = 0;
  for ($i=0; $i<8; $i++) for($j=0; $j<8; $j++)
     if (($mt[$k]->c = chkmov($b,$p,$i,$j))!=0){
      $mt[$k]->x =$i;
      $mt[$k]->y =$j;
      $mt[$k]->s = s_move($b,$p,$o,$e,$i,$j);
      ++$k;
    }
  return $k;
}

function analyze($b,$p,$o,$e)
{
  //struct mt  t[64];
  global $mt;
  global $a;

  $k = fillmt($b,$p,$o,$e);
  cpybrd($a,$b);
  for ($i=0; $i<$k; $i++){
    $c = ord('F') - $mt[$i]->s;
    if($c <= ord('Z')){
      $a[$mt[$i]->x][$mt[$i]->y] = chr($c);
    }
    else{
      $a[$mt[$i]->x][$mt[$i]->y] = 'Z';
    }
  }
  prtbrd($a);
}

function cmpmov($a,$b)
{
  global $mt;
  if ($mt[$a]->s > $mt[$b]->s) return -1;
  if ($mt[$a]->s < $mt[$b]->s) return 1;
  if ($mt[$a]->c > $mt[$b]->c) return -1;
  if ($mt[$a]->c < $mt[$b]->c) return 1;
  return 0;
}

function bsort($k)
{
  global $mt;
  global $mt_tmp;

  for($i=0;$i<$k-1;++$i){
    for($j=$i+1;$j<$k;++$j){
      if(cmpmov($i,$j) < 0){
        $mt_tmp = $mt[$i];
        $mt[$i] = $mt[$j];
        $mt[$j] = $mt_tmp;
      }
    }
  }
}

function my_mov($b,$p,$o,$e,&$game_i,&$game_j)
{
  global $mt;

  $k = fillmt($b,$p,$o,$e);
  if (!$k) return 0;
  bsort($k);
  for ($i=1; $i<$k; $i++)
    if ($mt[$i]->s != $mt[0]->s || $mt[$i]->c != $mt[0]->c)
            break;
  $k = rand() % $i;
  $game_i = $mt[$k]->x;
  $game_j = $mt[$k]->y;
  return 1;
}

function game(&$b,$n)
{
  global $his,$mine;
  global $mefirst;
  global $EMPTY;
  global $BLACK;
  global $WHITE;
  global $selfplay;
  global $handicap;
  global $h;

  $game_i = 0;
  $game_j = 0;

  $handicap = 0;
  $selfplay = ' ';
  $ff=0;

  if ($mefirst) {
    $mine = $BLACK; $his = $WHITE;
    printf("\nI go first:\n\n");
  }
  else {
    $mine = $WHITE; $his = $BLACK;
    printf("\nYou go first:\n\n");
  }

  while(1) {
    if (cntbrd($b,$EMPTY)==0) return 'D';
    if (cntbrd($b,$EMPTY)==60 && $mine == $BLACK) goto Istart;
    if (chkmvs($b,$his)==0) {
      printf(!$mefirst ? "Forfeit" : "   ...Forfeit\n");
      $ff |= 1;
    }
    else{
      $c = getmov($game_i,$game_j);
      if($c == 'B'){
        prtbrd($b); continue;
      }
      else
      if($c == 'S'){
        $game_i= prtscr($b);
        if ($game_i>0) printf(" You're winning\n");
        else if ($game_i<0)printf(" You're losing!\n");
        else putchar("\n");
        continue;
      }
      else
      if($c == 'Q'){ /* case 4: */
        return $c;
      }
      else
      if($c == 'H'){
        if ($n>abs($handicap)+4)
          printf("Illegal!\n");
         else for ($game_j=0; $game_i!=0 && $game_j<4; $game_j++) {
           $b[$h[$game_j][0]][$h[$game_j][1]]= $game_i>0?$BLACK:$WHITE;
           $handicap += $game_i>0 ? 1 : -1;
           ++$n;
           $game_i += $game_i>0 ? -1 : 1;
         }
         prtbrd($b); continue;
      }
      else
      if($c == 'A'){
        analyze($his,$mine,$EMPTY);
        continue;
      }
      else
      if($c == 'G'){
        my_mov($b,$his,$mine,$EMPTY,$game_i,$game_j);
      }
      else
      if($c == 'M'){
        if (chkmov($b,$his,$game_i,$game_j)>0) {
          printf(!$mefirst ? "%1d-%1d" : "   ...%1d-%1d\n",
            $game_i+1,$game_j+1);
          putmov($b,$his,$game_i,$game_j);
        }
        else {
          printf("Illegal!\n");
          continue;
        }
        goto Istart;
      }
      else
      if($c == 'F'){
        if ($n>abs($handicap)+4) {
          printf ("Illegal!\n");
          continue;
        }
        else printf(!$mefirst ? "Forfeit" :
             "   ...Forfeit\n");
      }
    }
Istart:
    if (cntbrd($b,$EMPTY) == 0) return 'D';
    if (chkmvs($b,$mine)==0) {
      printf(!$mefirst ? "...Forfeit\n": "Forfeit...\n");
      $ff |=2;
    }
    else {
      my_mov($b,$mine,$his,$EMPTY,$game_i,$game_j);
        printf(!$mefirst ? "...%1d-%1d\n" : "%1d-%1d...\n",
        $game_i+1,$game_j+1);
      putmov($b,$mine,$game_i,$game_j);
      ++$n;
    }
    if ($ff==3 || $n>64) return 'D';
    if (!($ff & 1)) prtbrd($b);
    $ff = 0;
  }
}

function ask($s)
{
  $c = ' ';
  printf ("%s ",$s);
  $a=getstring();
  return $a;
}

//void main(void /* int argc,char **argv */)
{
  $h[0][0] = $h[0][1] = $h[2][0] = $h[3][1] = 0;
  $h[1][0] = $h[1][1] = $h[2][1] = $h[3][0] = 7;
  printf("\nWelcome to the BDS C OTHELLO program!\n");
  printf("\nNote: `*' always goes first...Good luck!!!\n\n");

  printf("Do you want to go first? (y/n)");
  if (toupper(getchar()) == 'Y') 
    $mefirst = 0;
  else
    $mefirst = 1;

  printf("\n\n");

  do {
    clrbrd($b);
    prtbrd($b);
    $i = game($b,4);
    $mefirst = !$mefirst;
    if ($i==4) break;
    if ($i=='Q') continue;
    printf("\n");
    $i = prtscr($b);
    if ($i>0) printf(" You won by %d\n",$i);
    else if ($i<0) printf(" You lost by %d\n",-$i);
    else printf(" A draw\n");
  } while (ask("Another game? ")=='Y');
}

?>
